#ifndef OPEN_LISTS_PROBABILISTIC_OPEN_LIST_H
#define OPEN_LISTS_PROBABILISTIC_OPEN_LIST_H

#include "open_list.h"
#include "../option_parser.h"

#include <unordered_map>

class ScalarEvaluator;

template<class Entry>
class ProbabilisticOpenList : public OpenList<Entry>{
    typedef std::vector<Entry> Bucket;
    typedef std::unordered_map<std::pair<int, int>, Bucket, hash_int_pair> BucketList;

    ScalarEvaluator *evaluator;
    double probability_for_random_g;
    double h_probability_base;

    BucketList open_list;

    // A ref counting for the availible g and h values
    // this is a ordered map as we have to iterate the values in an ordered manner
    std::map<int, int> g_map;
    std::map<int, int> h_map;

    int last_h_value;
    int last_g_value;
    bool last_preferred;
    bool dead_end;
    bool dead_end_reliable;

protected:
    virtual ScalarEvaluator *get_evaluator() {return evaluator; }

public:
    explicit ProbabilisticOpenList(const Options &opts);
    ~ProbabilisticOpenList();

    virtual Entry remove_min(std::vector<int> *key);
    virtual int insert(const Entry &entry);

    virtual void evaluate(int g, bool preferred);

    virtual void get_involved_heuristics(std::set<Heuristic *> &hset);
    virtual bool dead_end_is_reliable() const;
    virtual bool is_dead_end() const;
    virtual void clear();
    virtual bool empty() const;

    static OpenList<Entry> *_parse(OptionParser &parser);
};


#include "probabilistic_open_list.cc"

// HACK! Need a better strategy of dealing with templates, also in the Makefile.

#endif
