// HACK! Ignore this if used as a top-level compile target.
#ifdef OPEN_LISTS_RANDOM_BUCKETS_OPEN_LIST_H

#include "../globals.h"

template<class Entry>
RandomBucketsOpenList<Entry>::RandomBucketsOpenList(const Options &opts)
    : StandardScalarOpenList<Entry>(opts),
      epsilon(opts.get<double>("epsilon")) {
}

template<class Entry>
RandomBucketsOpenList<Entry>::~RandomBucketsOpenList() {
}

template<class Entry>
Entry RandomBucketsOpenList<Entry>::remove_min(std::vector<int> *key) {
    assert(StandardScalarOpenList<Entry>::size > 0);
    if (g_rng() >= epsilon) {
        return StandardScalarOpenList<Entry>::remove_min(key);
    }
    size_t pos = g_rng(StandardScalarOpenList<Entry>::size);
    typename std::map<int, Bucket>::iterator it;
    it = StandardScalarOpenList<Entry>::buckets.begin();
    assert(it != StandardScalarOpenList<Entry>::buckets.end());
    int skipped_entries = 0;
    while (it != StandardScalarOpenList<Entry>::buckets.end() && skipped_entries + it->second.size() <= pos) {
        skipped_entries += it->second.size();
        ++it;
    }
    assert(it != StandardScalarOpenList<Entry>::buckets.end());
    Bucket &bucket = it->second;
    size_t pos_in_bucket = pos - skipped_entries;

    if (key) {
        assert(key->empty());
        key->push_back(it->first);
    }
    assert(pos_in_bucket < bucket.size());
    Entry result = bucket[pos_in_bucket];
    bucket.erase(bucket.begin() + pos_in_bucket);
    if (bucket.empty()) {
        StandardScalarOpenList<Entry>::buckets.erase(it);
    }
    --StandardScalarOpenList<Entry>::size;
    return result;
}

template<class Entry>
OpenList<Entry> *RandomBucketsOpenList<Entry>::_parse(OptionParser &parser) {
    parser.document_synopsis(
        "Random Buckets open list",
        "Random Buckets open list that uses a single evaluator. With probability epsilon the next entry is selected with a uniform random distribution, otherwise the minimum entry is chosen.");
    parser.add_option<ScalarEvaluator *>("eval", "scalar evaluator");
    parser.add_option<bool>(
        "pref_only",
        "insert only nodes generated by preferred operators", "false");
    parser.add_option<double>(
        "epsilon",
        "probability between 0 and 1 with which to choose the next entry randomly", "0.05");
    Options opts = parser.parse();

    if (parser.dry_run()) {
        return 0;
    } else {
        double epsilon = opts.get<double>("epsilon");
        if (epsilon > 1 || epsilon < 0) {
            parser.error("epsilon must be between 0 and 1");
        }
        return new RandomBucketsOpenList<Entry>(opts);
    }
}
#endif
